# -----------------------------------------------------------------------------
# Read PDB file atom coordinates
#
import os
import string

import atoms
import noesy
import pyutil
import tkutil

# -----------------------------------------------------------------------------
# Cache PDB Model objects
#
model_table = {}
def model(pdb_path, stoppable = tkutil.Not_Stoppable()):

  if not model_table.has_key(pdb_path):
    model_table[pdb_path] = pdb_model(pdb_path, stoppable)
  return model_table[pdb_path]

# -----------------------------------------------------------------------------
#
class pdb_model:

  def __init__(self, pdb_path, stoppable = tkutil.Not_Stoppable()):

    self.pdb_path = pdb_path
    self.pdb_atoms = self.read_pdb_atoms(pdb_path, stoppable)

    t = {}
    for a in self.pdb_atoms:
      t[(a.residue_number, a.atom_name)] = a
    self.atom_table = t

    gatable = {}
    for a in self.pdb_atoms:
      gatable[(a.residue_symbol, a.atom_name)] = 1
    self.atom_translations = \
      atoms.guess_atom_name_translations(pdb_path, gatable.keys())
    
  # ---------------------------------------------------------------------------
  #
  def read_pdb_atoms(self, pdb_path, stoppable):

    pdb_file = open(pdb_path, 'r')
    lines = pdb_file.readlines()
    pdb_file.close()

    atoms = []
    stoppable.stoppable_loop('reading ' + os.path.basename(pdb_path), 200)
    for line in lines:
      stoppable.check_for_stop()
      if line[:4] == 'ATOM':
	atoms.append(pdb_atom(line))

    return atoms

  # ---------------------------------------------------------------------------
  #
  def pdb_atom(self, residue_number, atom_name):

    try:
      return self.atom_table[(residue_number, atom_name)]
    except KeyError:
      return None

  # ---------------------------------------------------------------------------
  # Calculate distance between atoms.
  #
  def noesy_distance(self, peak):

    if peak.is_assigned:
      res = noesy.proton_resonances(peak)
      if res != None and len(res) == 2:
	translate = self.atom_translations
	na0 = translate.resonance_number_atom(res[0])
	na1 = translate.resonance_number_atom(res[1])
	return self.distance(na0, na1)
    return None

  # ---------------------------------------------------------------------------
  # Calculate distance between atoms.
  #
  def distance(self, num_atom_1, num_atom_2):

    atoms = self.atom_table
    if atoms.has_key(num_atom_1) and atoms.has_key(num_atom_2):
      return pyutil.distance(atoms[num_atom_1].xyz, atoms[num_atom_2].xyz)

    return None

  # ---------------------------------------------------------------------------
  #
  def find_resonance(self, pdb_atom, condition):

    mol_names = atoms.molecule_atom_name_translations(condition.molecule)
    gsym, aname = atoms.translate_group_atom(pdb_atom.residue_symbol,
                                             pdb_atom.atom_name,
                                             self.atom_translations, mol_names)
    gnum = pdb_atom.residue_number
    gname = gsym + str(gnum)
    return condition.find_resonance(gname, aname)

# -----------------------------------------------------------------------------
#
class pdb_atom:

  def __init__(self, line):

    self.atom_name = string.strip(line[12:16])
    self.residue_symbol = string.strip(line[16:20])
    self.residue_number = string.atoi(line[22:26])

    self.chain_id = line[21:22]
    self.xyz = (string.atof(line[30:38]),
		string.atof(line[38:46]),
		string.atof(line[46:54]))

  # ---------------------------------------------------------------------------
  #
  def full_name(self):
    return self.residue_symbol + str(self.residue_number) + self.atom_name
