# -----------------------------------------------------------------------------
# Strip plot display for 3D spectra
#

import string
import Tkinter

import axes
import pyutil
import sparky
import sputil
import tkutil

# -----------------------------------------------------------------------------
#
class strip_plot(tkutil.Dialog):

  def __init__(self):

    tkutil.Dialog.__init__(self, 'Strip Plot')

    self.strip_width = 60
    self.strip_gap = 8
    self.y_center = {'13C':None, '1H':None, '15N':None}
    self.y_pixel_size = {'13C':None, '1H':None, '15N':None}
    self.y_limits = {'13C':None, '1H':None, '15N':None}
    self.strips = []
    self.first_strip_index = 0
    self.choose_strips_dialog = choose_strips_dialog(self)

    mbar = self.create_menus(self.top)
    mbar.pack(side = 'top', anchor = 'nw', fill = 'x')

    sf = self.create_strip_frame(self.top)
    sf.pack(side = 'top', anchor = 'nw', fill = 'both', expand = 1)
    
    self.strips_frame.bind('<Configure>', self.resize_cb)
      
  # ---------------------------------------------------------------------------
  #
  def create_menus(self, parent):

    mbar = Tkinter.Frame(parent)

    m = tkutil.menu(mbar, 'Show')
    m.button.pack(side = 'left')

    m.add_button('Select strip spectra... (ss)', select_strip_spectra)
    sparky.add_command('ss', '', 'import strips; strips.select_strip_spectra()')
    m.add_separator()
    m.add_button('Add selected peak strips (sk)', add_peak_strips)
    sparky.add_command('sk', '', 'import strips; strips.add_peak_strips()')
    m.add_button('All assigned strips (sn)', add_assigned_strips)
    sparky.add_command('sn', '', 'import strips; strips.add_assigned_strips()')
    m.add_separator()
    m.add_button('Copy view strip options (sv)', copy_view_options)
    sparky.add_command('sv', '', 'import strips; strips.copy_view_options()')
    m.add_separator()
    m.add_button('Delete selected strip (sd)', delete_view_strip)
    sparky.add_command('sd', '', 'import strips; strips.delete_view_strip()')
    m.add_button('Delete all strips (sD)', delete_all_strips)
    sparky.add_command('sD', '', 'import strips; strips.delete_all_strips()')
    m.add_separator()
    m.add_button('Help', sputil.help_cb('StripPlot'))
    m.add_button('Close', self.close_cb)
      
    m = tkutil.menu(mbar, 'Find')
    m.button.pack(side = 'left')

    m.add_button('Go to assigned peak strip (sj)', goto_assigned_strip)
    sparky.add_command('sj', '', 'import strips; strips.goto_assigned_strip()')
    m.add_separator()
    m.add_button('Add strips matching peaks (sm)', add_matching_strips)
    sparky.add_command('sm', '', 'import strips; strips.add_matching_strips()')
    m.add_button('Delete matched strips (sM)', delete_matched_strips)
    sparky.add_command('sM', '', 'import strips; strips.delete_matched_strips()')

    return mbar
  
  # ---------------------------------------------------------------------------
  #
  def create_strip_frame(self, parent):

    sf = Tkinter.Frame(parent)
    
    sf.rowconfigure(0, weight = 1)
    sf.columnconfigure(0, weight = 1)

    strips = Tkinter.Frame(sf, width = 200, height = 800)
    strips.grid(row = 0, column = 0, sticky = 'news')
    strips.rowconfigure(0, weight = 1)
    self.strips_frame = strips
    
    vbar = Tkinter.Scrollbar(sf, command = self.scroll_strips)
    vbar.grid(row = 0, column = 1, sticky = 'ns')
    vbar.set(0, 1)
    self.vbar = vbar
    
    hbar = Tkinter.Scrollbar(sf, orient = 'horizontal',
                             command = self.shift_strips)
    hbar.grid(row = 1, column = 0, sticky = 'ew')
    hbar.set(0, 1)
    self.hbar = hbar

    return sf

  # ---------------------------------------------------------------------------
  #
  def resize_cb(self, event):
    
    self.change_visible_strips()
    self.update_vertical_scrollbar()
    
  # ---------------------------------------------------------------------------
  #
  def add_strips(self, strips):

    for strip in strips:
      self.initial_y_scale(strip)
      self.strips.append(strip)

    if strips:
      self.change_visible_strips()
  
  # ---------------------------------------------------------------------------
  #
  def sort_strips(self, compare):

    if self.strips:

      first = self.strips[self.first_strip_index]
      self.strips.sort(compare)
      self.first_strip_index = self.strips.index(first)
      self.change_visible_strips()
  
  # ---------------------------------------------------------------------------
  #
  def goto_strip(self, strip):

    s = self.strip_position(strip)
    if s == None:
      return

    self.first_strip_index = s
    self.change_visible_strips()
  
  # ---------------------------------------------------------------------------
  #
  def strip_position(self, strip):

    for s in range(len(self.strips)):
      if self.strips[s] == strip:
        return s
    return None
      
  # ---------------------------------------------------------------------------
  # Set the y pixel size and center y position for the strip
  #
  def initial_y_scale(self, strip):

    y_nucleus = strip.y_nucleus
    y_limits = self.y_limits[y_nucleus]
      
    if y_limits == None:

      y_axis = strip.xyz_axes[1]
      r = strip.spectrum.region
      y_min = r[0][y_axis]
      y_max = r[1][y_axis]
      y_middle = .5 * (y_min + y_max)
      y_range = y_max - y_min

      self.strips_frame.update_idletasks()        # determine window geometry
      window_height = self.strips_frame.winfo_height()

      self.y_limits[y_nucleus] = (y_min, y_max)
      self.y_center[y_nucleus] = y_middle
      self.y_pixel_size[y_nucleus] = y_range / window_height

    strip.set_y_center(self.y_center[y_nucleus])
    strip.set_y_pixel_size(self.y_pixel_size[y_nucleus])
    
  # ---------------------------------------------------------------------------
  #
  def delete_strips(self, strips):

    deleted_strips = {}
    for strip in strips:
      deleted_strips[strip] = 1

    keepers = []
    for s in range(len(self.strips)):
      strip = self.strips[s]
      if deleted_strips.has_key(strip):
        strip.set_column(self.strips_frame, None)
        if s < self.first_strip_index:
          self.first_strip_index = self.first_strip_index - 1
      else:
        keepers.append(strip)
    self.strips = keepers
    self.change_visible_strips()
    
  # ---------------------------------------------------------------------------
  #
  def delete_view_strip(self, view):

    for strip in self.strips:
      if strip.view == view:
        self.delete_strips([strip])
        return
    
  # ---------------------------------------------------------------------------
  #
  def delete_all_strips(self):

    for i in range(len(self.strips)):
      self.strips[i].set_column(self.strips_frame, None)
    self.strips = []
    self.first_strip_index = 0
    for nucleus in self.y_limits.keys():
      self.y_limits[nucleus] = None
    self.change_visible_strips()

  # ---------------------------------------------------------------------------
  #
  def scroll_strips(self, *args):

    tkutil.adjust_scrollbar(self.vbar, args)
    (f1, f2) = self.vbar.get()
    f = .5 * (f1 + f2)
    for strip in self.strips:
      y_limits = self.y_limits[strip.y_nucleus]
      self.y_center[strip.y_nucleus] = f * y_limits[1] + (1-f) * y_limits[0]
      strip.set_y_center(self.y_center[strip.y_nucleus])

  # ---------------------------------------------------------------------------
  #
  def shift_strips(self, *args):

    steps_per_page = self.visible_strip_count()
    tkutil.adjust_scrollbar(self.hbar, args, steps_per_page)
    (f1, f2) = self.hbar.get()
    self.first_strip_index = int(round(f1 * len(self.strips)))
    self.change_visible_strips()
    
  # ---------------------------------------------------------------------------
  #
  def change_visible_strips(self):

    first_index = self.first_strip_index
    for strip in self.strips[:first_index]:
      strip.set_column(self.strips_frame, None)
    count = self.visible_strip_count()
    for strip in self.strips[first_index+count:]:
      strip.set_column(self.strips_frame, None)
    for i in range(count):
      self.strips[first_index + i].set_column(self.strips_frame, i)
    self.update_horizontal_scrollbar()

  # ---------------------------------------------------------------------------
  #
  def update_vertical_scrollbar(self):
    
    if self.first_strip_index < len(self.strips):
      strip = self.strips[self.first_strip_index]
      if strip.view:
        y_axis = strip.xyz_axes[1]
        sr = strip.spectrum.region
        r_low = sr[0][y_axis]
        r_high = sr[1][y_axis]
        self.top.update_idletasks()      # Make sure view geometry is correct
        vr = strip.view.region
        y_low = vr[0][y_axis]
        y_high = vr[1][y_axis]
        tkutil.set_scrollbar(self.vbar, y_low, y_high, r_low, r_high)

  # ---------------------------------------------------------------------------
  #
  def update_horizontal_scrollbar(self):

    self.top.update_idletasks()      # Make sure view geometry is correct

    first = self.first_strip_index
    count = self.visible_strip_count()
    tkutil.set_scrollbar(self.hbar, first, first + count, 0, len(self.strips))

  # ---------------------------------------------------------------------------
  #
  def visible_strip_count(self):

    frame_width = self.strips_frame.winfo_width()
    max_strips = len(self.strips) - self.first_strip_index
    count = max(1, frame_width / self.strip_width)
    return min(max_strips, count)
  
  # ---------------------------------------------------------------------------
  #
  def spectrum_strip(self, spectrum, center, label_text):

    xyz_axes = self.choose_strips_dialog.xyz_axes(spectrum)
    if xyz_axes == None:
      return None

    settings = self.default_view_options()
    mimic_view = self.view_to_mimic(spectrum)
    if mimic_view:
      for attr in settings.keys():
        settings[attr] = getattr(mimic_view, attr)

    s = strip(self.strip_width, self.strip_gap,
              spectrum, xyz_axes, center, settings, label_text)

    return s
  
  # ---------------------------------------------------------------------------
  #
  def default_view_options(self):

    return {
      'pixel_size':None,
      'visible_depth':None,
      'contour_levels':None,
      'show_ornaments':1,
      'show_peaks':1,
      'show_peakgroups':1,
      'show_labels':1,
      'show_lines':1,
      'show_grids':1,
      'show_crosshair':1,
      'show_crosshair_from_other_views':1,
      }

  # ---------------------------------------------------------------------------
  #
  def view_to_mimic(self, spectrum):

    for strip in self.strips:
      if strip.view and strip.spectrum == spectrum:
        return strip.view
    return sputil.preferred_view(spectrum)
    
  # ---------------------------------------------------------------------------
  #
  def copy_view_options(self, view):

    if view:

      settings = self.default_view_options()
      for attr in settings.keys():
        settings[attr] = getattr(view, attr)

      for strip in self.strips:
        if strip.spectrum == view.spectrum:
          strip.change_view_options(settings)
    
# -----------------------------------------------------------------------------
#
class strip:

  def __init__(self, strip_width, strip_gap,
               spectrum, xyz_axes, center, settings,
               label_text):

    self.strip_width = strip_width
    self.strip_gap = strip_gap
    self.spectrum = spectrum
    self.xyz_axes = xyz_axes
    self.y_nucleus = spectrum.nuclei[xyz_axes[1]]
    self.center = center
    self.settings = settings.copy()
    self.label_text = label_text

    self.view = None
    self.label = None

  # ---------------------------------------------------------------------------
  #
  def __del__(self):

    self.delete_label()
    self.delete_view()

  # ---------------------------------------------------------------------------
  #
  def set_column(self, parent, column):

    self.position_label(parent, column)
    self.request_view_position(parent, column)
    
  # ---------------------------------------------------------------------------
  # Defer view creation and deletion so scrolling through many strips
  # remains responsive.
  #
  views_to_update = {}
  def request_view_position(self, parent, column):
    self.views_to_update[self] = (parent, column)
    if len(self.views_to_update) == 1:
      parent.after_idle(self.update_views)
  def update_views(self):
    if self.views_to_update:
      strip, (parent, column) = self.views_to_update.items()[0]
      del self.views_to_update[strip]
      strip.position_view(parent, column)
      if self.views_to_update:
        parent.after_idle(self.update_views)
        
  # ---------------------------------------------------------------------------
  #
  def position_label(self, parent, column):

    if column == None:
      self.delete_label()
    else:
      parent.columnconfigure(column, minsize = self.strip_width)
      self.create_label(parent, column)
      self.label.grid(row = 1, column = column, sticky = 'news')
        
  # ---------------------------------------------------------------------------
  #
  def create_label(self, parent, column):

    if self.label == None:
      color = sputil.spectrum_color(self.spectrum)
      self.label = Tkinter.Label(parent, text = self.label_text,
                                 highlightthickness = 1,
                                 highlightbackground = color)

  # ---------------------------------------------------------------------------
  #
  def delete_label(self):

    if self.label:
      self.label.destroy()
      self.label = None

  # ---------------------------------------------------------------------------
  #
  def position_view(self, parent, column):

    if column == None:
      self.delete_view()
    else:
      if self.view == None:
        self.view = self.create_view(parent, column)
      parent.tk.call('grid', 'configure', self.view.frame,
                     '-row', 0, '-column', column,
                     '-sticky', 'news', '-padx', self.strip_gap/2)

  # ---------------------------------------------------------------------------
  #
  def create_view(self, parent, column):

    v = sparky.View(parent, self.spectrum)
    v.show_scales = 0
    v.show_scrollbars = 0
    v.axis_order = self.xyz_axes
    v.center = self.center

    self.set_view_options(v, self.settings)

    #
    # Set the view frame width and don't listen to resize requests from view
    #
    width = self.strip_width - self.strip_gap
    height = 800
    color = sputil.spectrum_color(self.spectrum)
    tkutil.tk_call(v.frame, 'configure', '-width', width, '-height', height,
                   '-highlightthickness', 2, '-highlightcolor', color)
    tkutil.tk_call('grid', 'propagate', v.frame, 0)

    return v
  
  # ---------------------------------------------------------------------------
  #
  def change_view_options(self, settings):

    self.settings = settings.copy()
    if self.view:
      self.set_view_options(self.view, settings)
      
  # ---------------------------------------------------------------------------
  #
  def set_view_options(self, view, settings):
    
    for attr, value in settings.items():
      if value == None:
        settings[attr] = getattr(view, attr)
      else:
        setattr(view, attr, value)
  
  # ---------------------------------------------------------------------------
  #
  def delete_view(self):

    if self.view:
      for attr in self.settings.keys():
        self.settings[attr] = getattr(self.view, attr)
      self.view.destroy()
      self.view = None
    
  # ---------------------------------------------------------------------------
  #
  def set_y_center(self, y_center):
    
    center = list(self.center)
    y_axis = self.xyz_axes[1]
    center[y_axis] = y_center
    self.center = tuple(center)
    if self.view:
      self.view.center = self.center

  # ---------------------------------------------------------------------------
  #
  def set_y_pixel_size(self, y_pixel_size):

    y_axis = self.xyz_axes[1]
    pixel_size = self.settings['pixel_size']
    factor = y_pixel_size / pixel_size[y_axis]
    pixel_size = pyutil.scale_tuple(pixel_size, factor)
    self.settings['pixel_size'] = pixel_size
    if self.view:
      self.view.pixel_size = pixel_size
    
# -----------------------------------------------------------------------------
# Show strips for prespecified spectra for selected 2D or 3D peaks.
#
class choose_strips_dialog(tkutil.Dialog):

  def __init__(self, strip_dialog):

    self.strip_dialog = strip_dialog
    
    tkutil.Dialog.__init__(self, 'Show Peak Strips')

    f = self.spectrum_choice_table(self.top)
    f.pack(side = 'top', anchor = 'w')

    f = self.matching_parameters(self.top)
    f.pack(side = 'top', anchor = 'w')

    self.msg = Tkinter.Label(self.top)
    self.msg.pack(side = 'top', anchor = 'w')
    
    br = tkutil.button_row(self.top,
			   ('Close', self.close_cb),
                           ('Help', sputil.help_cb('StripPlot')),
			   )
    br.frame.pack(side = 'top', anchor = 'w')

  # ---------------------------------------------------------------------------
  #
  def spectrum_choice_table(self, parent):

    f = Tkinter.Frame(parent, borderwidth = 5)

    lbl = Tkinter.Label(f, text = 'Show strips')
    lbl.grid(row = 0, column = 0)

    lbl = Tkinter.Label(f, text = 'xyz axis order')
    lbl.grid(row = 0, column = 1)
    
    self.chosen_spectra = []
    self.axis_order_menu = {}
    count = 0
    for spectrum in sputil.spectrum_list():
      if spectrum.dimension == 3:
        count = count + 1
        cb = tkutil.checkbutton(f, spectrum.name, 0)
        cb.button['selectcolor'] = sputil.spectrum_color(spectrum)
        choose_cb = pyutil.precompose(sputil.choose_spectrum_cb,
                                      spectrum, self.chosen_spectra)
        cb.add_callback(choose_cb)
        cb.button.grid(row = count, column = 0, sticky = 'w')

        chn_axes = axes.chn_axes(spectrum)
        if chn_axes:
          order = (chn_axes[1], chn_axes[0], chn_axes[2])
        else:
          order = None
        aom = axes.axis_order_menu(f, spectrum, order)
        aom.frame.grid(row = count, column = 1)
        self.axis_order_menu[spectrum] = aom

    return f

  # ---------------------------------------------------------------------------
  #
  def matching_parameters(self, parent):

    f = Tkinter.Frame(parent, borderwidth = 5)

    lbl = Tkinter.Label(f, text = 'Strip matching parameters')
    lbl.pack(side = 'top', anchor = 'w')

    t = tkutil.entry_row(f, 'Tolerances ',
                         (' 1H', '.01', 5),
                         (' 13C', '.1', 5),
                         (' 15N', '.1', 5))
    t.frame.pack(side = 'top', anchor = 'w')
    self.tolerances = t.variables

    e = tkutil.entry_field(f, 'Allow ', '0', 3, ' unmatched peaks')
    e.frame.pack(side = 'top', anchor = 'w')
    self.max_unmatched = e.variable

    return f
  
  # ---------------------------------------------------------------------------
  #
  def xyz_axes(self, spectrum):

    if not self.axis_order_menu.has_key(spectrum):
      return None
    return self.axis_order_menu[spectrum].axis_order()
    
  # ---------------------------------------------------------------------------
  #
  def message(self, text):

    self.msg['text'] = text
    
  # ---------------------------------------------------------------------------
  #
  def show_peak_strips(self, peaks):

    strips = self.strips_for_peaks(peaks)
    self.strip_dialog.show_window(1)
    self.strip_dialog.add_strips(strips)
    self.message('Added %d strips for %d peaks' % (len(strips), len(peaks)))
    
  # ---------------------------------------------------------------------------
  #
  def strips_for_peaks(self, peaks):

    spectra_3d = self.chosen_spectra

    spectra = spectra_3d
    strips = []
    for peak in peaks:
      if not spectra_3d:
        spectra = [peak.spectrum]
      peak_strips = self.peak_strips(peak, spectra)
      for strip in peak_strips:
        strips.append(strip)

    return strips
    
  # ---------------------------------------------------------------------------
  #
  def peak_strips(self, peak, spectra_3d):

    peak_res = peak.resonances()
    previous_label = ''
    strips = []
    for spectrum in spectra_3d:
      freq = [0,0,0]
      aX, aY, aZ = self.xyz_axes(spectrum)
      peak_aX = axes.unique_nucleus_axis(peak.spectrum, spectrum.nuclei[aX])
      peak_aZ = axes.unique_nucleus_axis(peak.spectrum, spectrum.nuclei[aZ])
      if peak_aX != None and peak_aZ != None:
        freq[aX] = peak.frequency[peak_aX]
        freq[aZ] = peak.frequency[peak_aZ]
        label = '%.4g\n%.4g' % (freq[aX], freq[aZ])
        if peak_res[peak_aX]:
          label = label + '\n' + peak_res[peak_aX].group.name
        if label == previous_label:
          label = ''
        center = sputil.alias_onto_spectrum(freq, spectrum)
        strip = self.strip_dialog.spectrum_strip(spectrum, center, label)
        strips.append(strip)
        if label:
          previous_label = label

    return strips
    
  # ---------------------------------------------------------------------------
  #
  def add_assigned_strips(self):

    spectra_3d = self.chosen_spectra

    xz_assignments = {}
    for spectrum in spectra_3d:
      aX, aY, aZ = self.xyz_axes(spectrum)
      for peak in spectrum.peak_list():
        res = peak.resonances()
        rX = res[aX]
        rZ = res[aZ]
        if rX and rZ:
          xz_assignments[((rX.group.number, rX.group.name, rX.atom.name),
                          (rZ.group.number, rZ.group.name, rZ.atom.name))] = 1
    xz_assignments = xz_assignments.keys()
    xz_assignments.sort()
    xz_assignments = map(lambda xz: (xz[0][1:], xz[1][1:]), xz_assignments)
    
    strips = []
    for xz_assignment in xz_assignments:
      for spectrum in spectra_3d:
        strip = self.assigned_strip(spectrum, xz_assignment)
        if strip:
          strips.append(strip)
    self.strip_dialog.add_strips(strips)
    
  # ---------------------------------------------------------------------------
  #
  def assigned_strip(self, spectrum, xz_assignment):

    c = spectrum.condition
    x_assignment, z_assignment = xz_assignment
    rX = apply(c.find_resonance, x_assignment)
    rZ = apply(c.find_resonance, z_assignment)
    if rX and rZ:
      freq = [0,0,0]
      aX, aY, aZ = self.xyz_axes(spectrum)
      freq[aX] = rX.frequency
      freq[aZ] = rZ.frequency
      center = sputil.alias_onto_spectrum(freq, spectrum)
      group_name = x_assignment[0]
      label = '%.4g\n%.4g\n%s' % (center[aX], center[aZ], group_name)
      strip = self.strip_dialog.spectrum_strip(spectrum, center, label)
      strip.xz_assignment = xz_assignment
      return strip
    return None
    
  # ---------------------------------------------------------------------------
  #
  def goto_assigned_strip(self, peak):

    assignments = []
    for r in peak.resonances():
      if r:
        assignments.append((r.group.name, r.atom.name))

    if len(assignments) < 2:
      return
    
    for strip in self.strip_dialog.strips:
      if hasattr(strip, 'xz_assignment'):
        x_assignment, z_assignment = strip.xz_assignment
        if x_assignment in assignments and z_assignment in assignments:
          self.strip_dialog.goto_strip(strip)
          return
    
  # ---------------------------------------------------------------------------
  #
  def add_matching_strips(self, spectrum, target_peaks):

    if spectrum == None:
      return

    h_tol, c_tol, n_tol = tkutil.float_variable_values(self.tolerances)
    match_tolerances = {'1H':h_tol, '15N':n_tol, '13C':c_tol}
    tolerances = []
    for nucleus in spectrum.nuclei:
      tolerances.append(match_tolerances[nucleus])
    
    target_shifts = []
    for peak in target_peaks:
      xyz_axes = self.xyz_axes(peak.spectrum)
      if xyz_axes:
        aX, aY, aZ = xyz_axes
        target_shifts.append(peak.frequency[aY])

    max_unmatched = string.atoi(self.max_unmatched.get())

    strips = self.matching_strips(spectrum, tolerances,
                                  target_shifts, max_unmatched)
    self.strip_dialog.add_strips(strips)
    
  # ---------------------------------------------------------------------------
  #
  def matching_strips(self, spectrum, tolerances,
                      target_shifts, max_unmatched):

    xyz_axes = self.xyz_axes(spectrum)
    if xyz_axes == None:
      return []
    aX, aY, aZ = xyz_axes

    matches = {}
    clusters = sputil.cluster_peaks(spectrum.peak_list(),
                                    aX, tolerances[aX], aZ, tolerances[aZ])
    for cluster in clusters:
      score = self.match_score(cluster, aY, tolerances[aY],
                               target_shifts, max_unmatched)
      if score > 0:
        matches[cluster[0]] = score
    peaks = pyutil.sort_keys_by_value(matches)
    peaks.reverse()                     # put in descending order

    strips = []
    for peak in peaks:
      strip = self.peak_strips(peak, [spectrum])[0]
      strips.append(strip)

    self.last_match_strips = strips     # for deleting
    
    return strips
    
  # ---------------------------------------------------------------------------
  #
  def match_score(self, cluster, axis, tolerance,
                  target_shifts, max_unmatched):

    no_match = 0
    score = 0
    for target in target_shifts:
      range = tolerance
      for peak in cluster:
        freq = peak.frequency[axis]
        delta = abs(freq - target)
        range = min(delta, range)
      if range == tolerance:
        no_match = no_match + 1
        if no_match > max_unmatched:
          return 0
      else:
        score = score + (tolerance - range)
    return score
    
  # ---------------------------------------------------------------------------
  #
  def delete_matched_strips(self):

    if hasattr(self, 'last_match_strips'):
      self.strip_dialog.delete_strips(self.last_match_strips)
      self.last_match_strips = []

# -----------------------------------------------------------------------------
#
def strip_dialog():
  return tkutil.the_dialog(strip_plot)
def choose_strips():
  return strip_dialog().choose_strips_dialog
  
# -----------------------------------------------------------------------------
# Strip plot commands
#
def show_strip_plot():
  strip_dialog().show_window(1)

def delete_view_strip():
  strip_dialog().delete_view_strip(sparky.selected_view())
def delete_all_strips():
  strip_dialog().delete_all_strips()
def copy_view_options():
  strip_dialog().copy_view_options(sparky.selected_view())

def select_strip_spectra():
  choose_strips().show_window(1)
def add_peak_strips():
  choose_strips().show_peak_strips(sparky.selected_peaks())
def add_matching_strips():
  choose_strips().add_matching_strips(sputil.selected_spectrum(),
                                      sparky.selected_peaks())
def delete_matched_strips():
  choose_strips().delete_matched_strips()

def add_assigned_strips():
  choose_strips().add_assigned_strips()
def goto_assigned_strip():
  choose_strips().goto_assigned_strip(sputil.selected_peak())
